/*
* Author:          Patrick-Christopher Mattulat
* Company:         Lynar Studios
* E-Mail:          webmaster@lynarstudios.com
* Created:         2024-05-16
* Changed:         2024-05-17
*
* */

#ifndef LS_STD_EVENT_MANAGER_REWORKED_HPP
#define LS_STD_EVENT_MANAGER_REWORKED_HPP

#include "Event.hpp"
#include "EventListener.hpp"
#include <list>
#include <ls-std/core/Class.hpp>
#include <ls-std/event/reworked/type/EventTypes.hpp>
#include <ls-std/os/dynamic-goal.hpp>
#include <memory>
#include <unordered_map>

namespace ls::std::event::reworked::type
{
  using event_listeners = ::std::list<::std::pair<::std::shared_ptr<ls::std::event::reworked::EventListener>, ls::std::event::reworked::type::event_action>>;
  using event_observability_inventory = ::std::unordered_map<::std::string, ls::std::event::reworked::type::event_listeners>;
}

namespace ls::std::event::reworked
{
  class LS_STD_DYNAMIC_GOAL EventManager : public ls::std::core::Class
  {
    public:

      explicit EventManager();
      ~EventManager() noexcept override;

      [[nodiscard]] ls::std::event::reworked::type::listener_id getNextProvisionId() const;
      [[nodiscard]] bool holdsListenerForEvent(ls::std::event::reworked::type::listener_id _id, const ls::std::event::reworked::Event &_event);
      void invoke(const ls::std::event::reworked::Event &_event) const;
      [[nodiscard]] ls::std::event::reworked::type::listener_id requestListenerId();
      void subscribeListenerForEvent(::std::shared_ptr<ls::std::event::reworked::EventListener> _listener, const ls::std::event::reworked::Event &_event, ls::std::event::reworked::type::event_action _action);

    private:

      ls::std::event::reworked::type::event_observability_inventory inventory{};
      ls::std::event::reworked::type::listener_id provisionId = 1;

      static void _notifyListeners(const ls::std::event::reworked::type::event_listeners &_listeners);
      bool _observesEvent(const ls::std::event::reworked::Event &_event) const;
  };
}

#endif
